"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var symbol_observable_1 = __importDefault(require("symbol-observable"));
var lifecycle_1 = require("../../events/lifecycle");
var constants_1 = require("../../helpers/constants");
var isModule_1 = require("../../helpers/is/isModule/isModule");
var uniqify_1 = require("../../helpers/uniqueId/uniqify");
var bindApi_1 = require("./bindApi");
var getReadyPromise_1 = require("./getReadyPromise");
var getStore_1 = require("./getStore");
var setObservableConfig_1 = require("./setObservableConfig");
/**
 * Creates an application and returns a [[Stapp]].
 * Stage I
 *  1) Get module (call ModuleFactory if needed)
 *  2) Collect dependencies
 *  3) Combine reducers
 *  4) Collect waitFor
 * Stage II
 *  1) Check dependencies
 *  2) Create store
 * Stage II
 *  1) Connect epics
 *  2) Connect api
 * @param config createApp config
 * @param config.name Application name
 * @param config.modules Array of modules or module factories
 * @param config.dependencies Argument that will be passed to every module factory
 * @param config.rehydrate Value will be passed as an initialState to redux createStore
 */
exports.createApp = function (config) {
    var _a;
    var appName = config.name || uniqify_1.uniqify('Stapp');
    var devtools = config.devtools !== false
        ? Object.assign({
            name: appName
        }, config.devtools)
        : false;
    var anyModules = config.modules;
    var dependencies = config.dependencies || {};
    var initialState = config.rehydrate || {};
    var middlewares = config.middlewares || [];
    var handleEpicsErrors = config.handleEpicsErrors;
    var modules = [];
    var moduleNames = new Set();
    var moduleDependencies = new Set();
    var reducers = {};
    var api = {};
    var waitFor = [];
    var subscriptions = [];
    var _loop_1 = function (anyModule) {
        var module_1 = isModule_1.isModule(anyModule) ? anyModule : anyModule(dependencies);
        modules.push(module_1);
        if (!module_1.name) {
            throw new Error(constants_1.APP_KEY + " error: Module name is not provided");
        }
        if (moduleNames.has(module_1.name)) {
            throw new Error(constants_1.APP_KEY + " error: Module name should be unique");
        }
        moduleNames.add(module_1.name);
        if (module_1.dependencies) {
            module_1.dependencies.forEach(function (dependency) {
                return moduleDependencies.add(dependency);
            });
        }
        var moduleState = module_1.state || module_1.reducers;
        if (moduleState) {
            Object.keys(moduleState).forEach(function (stateKey) {
                reducers[stateKey] = moduleState[stateKey];
            });
        }
        if (module_1.waitFor) {
            waitFor = waitFor.concat(module_1.waitFor);
        }
    };
    for (var _i = 0, anyModules_1 = anyModules; _i < anyModules_1.length; _i++) {
        var anyModule = anyModules_1[_i];
        _loop_1(anyModule);
    }
    moduleNames.forEach(function (moduleName) { return moduleDependencies.delete(moduleName); });
    if (moduleDependencies.size !== 0) {
        throw new Error(constants_1.APP_KEY + " error: Please, provide dependencies: " + Array.from(moduleDependencies).join(', '));
    }
    var store = getStore_1.getStore(reducers, initialState, middlewares, devtools);
    var _loop_2 = function (module_2) {
        var epics = module_2.epic || module_2.epics;
        var events = module_2.api || module_2.events;
        if (!epics && !events) {
            return "continue";
        }
        var dispatch = store.createDispatch(module_2.name);
        if (epics) {
            var epicsArray = Array.isArray(epics) ? epics : [epics];
            var _a = setObservableConfig_1.getConfig(module_2), fromESObservable_1 = _a.fromESObservable, toESObservable_1 = _a.toESObservable;
            epicsArray
                .map(function (epic) {
                return epic(fromESObservable_1(store.event$), fromESObservable_1(store.state$), {
                    dispatch: dispatch,
                    getState: store.getState,
                    fromESObservable: fromESObservable_1,
                    toESObservable: toESObservable_1
                });
            })
                .filter(function (epicStream) { return !!epicStream; })
                .forEach(function (epicStream) {
                subscriptions.push(toESObservable_1(epicStream).subscribe(dispatch, handleEpicsErrors));
            });
        }
        if (events) {
            var moduleApi_1 = bindApi_1.bindApi(events, dispatch);
            Object.keys(moduleApi_1).forEach(function (apiKey) {
                api[apiKey] = moduleApi_1[apiKey];
            });
        }
    };
    for (var _b = 0, modules_1 = modules; _b < modules_1.length; _b++) {
        var module_2 = modules_1[_b];
        _loop_2(module_2);
    }
    var readyPromise = getReadyPromise_1.getReadyPromise(store.event$, store.getState, waitFor);
    var rootDispatch = store.createDispatch('root');
    store.flushQueue();
    rootDispatch(lifecycle_1.initEvent());
    var disconnect = function () {
        rootDispatch(lifecycle_1.disconnectEvent());
        store.disconnect();
        subscriptions.forEach(function (subscription) {
            subscription.unsubscribe();
        });
    };
    // tslint:disable-next-line no-floating-promises
    readyPromise.then(function () { return rootDispatch(lifecycle_1.readyEvent()); });
    return Object.freeze((_a = {
            name: appName,
            subscribe: function (next) {
                return store.state$.subscribe(next);
            },
            dispatch: rootDispatch,
            getState: store.getState,
            ready: readyPromise,
            api: api,
            disconnect: disconnect
        },
        _a[symbol_observable_1.default] = function () {
            return this;
        },
        _a));
};
//# sourceMappingURL=createApp.js.map